@@ -1,5 +1,8 @@
 # -- Array to array -------------------
 codec_transpose_encode <- function(array, indices = seq_along(dim(array))) {
+  if (identical(indices, seq_along(dim(array)))) {
+    return(array)
+  }
   dim(array) <- dim(array)[indices]
   array <- aperm(array, indices)
 
@@ -7,6 +10,9 @@ codec_transpose_encode <- function(array, indices = seq_along(dim(array))) {
 }
 
 codec_transpose_decode <- function(array, indices = seq_along(dim(array))) {
+  if (identical(indices, seq_along(dim(array)))) {
+    return(array)
+  }
   inv_indices <- order(indices)
   array <- aperm(array, inv_indices)
   dim(array) <- dim(array)[inv_indices]
@@ -16,11 +22,12 @@ codec_transpose_decode <- function(array, indices = seq_along(dim(array))) {
 
 # -- Array to bytes -------------------
 codec_endian_encode <- function(raw_obj, endian, bytesize) {
-  if (is.na(endian) || identical(endian, .Platform$endian)) {
+  if (is.na(endian) || endian == .Platform$endian) {
     return(raw_obj)
   }
 
-  ind <- rep_len(rev(seq_len(bytesize)), length(raw_obj)) + (seq_along(raw_obj) - 1) %/% bytesize * bytesize
+  ind <- rep_len(rev(seq_len(bytesize)), length(raw_obj)) +
+    (seq_along(raw_obj) - 1) %/% bytesize * bytesize
   return(raw_obj[ind])
 }
 
