@@ -47,22 +47,45 @@ read_zarr_array <- function(zarr_array_path, index, s3_client) {
     s3_client <- .create_s3_client(path = zarr_array_path)
   }
 
-  metadata_file <- .file_or_blob_exists(
+  metadata_files <- .file_or_blob_exists(
     zarr_array_path,
     s3_client,
     c(".zarray", "zarr.json")
   )
 
-  if (metadata_file["zarr.json"]) {
-    stop("Reading Zarr v3 arrays is not currently supported", call. = FALSE)
+  if (metadata_files[".zarray"] && metadata_files["zarr.json"]) {
+    stop(
+      "The path contains both `.zarray` (Zarr V2 specification) and ",
+      "`zarr.json` (Zarr V3 specification) metadata files.\n",
+      "An array or group must conform to either the Zarr V2 or V3 ",
+      "specification.",
+      call. = FALSE
+    )
+  }
+  if (!any(metadata_files)) {
+    stop(
+      "The path does not contain any metadata files. ",
+      "It must contain one of: ",
+      "  - `.zarray` (Zarr V2 specification)\n",
+      "  - `zarr.json` (Zarr V3 specification)",
+      call. = FALSE
+    )
   }
 
   metadata <- .read_array_metadata(
     zarr_array_path,
-    ".zarray",
+    names(metadata_files)[metadata_files],
     s3_client = s3_client
   )
 
+  if (metadata$zarr_format == 2) {
+    metadata <- .convert_metadata_version(
+      metadata,
+      version_from = 2,
+      version_to = 3
+    )
+  }
+
   ## if no index provided we will return everything
   if (missing(index)) {
     index <- vector(mode = "list", length = length(metadata$shape))
@@ -85,20 +108,22 @@ read_zarr_array <- function(zarr_array_path, index, s3_client) {
 }
 
 .extract_elements <- function(
-  i,
+  current_chunk_index,
+  current_chunk_name,
   metadata,
   index,
-  required_chunks,
   zarr_array_path,
   s3_client,
   chunk_idx
 ) {
   ## find elements to select from the chunk and what in the output we replace
   index_in_result <- index_in_chunk <- list()
-  alt_chunk_dim <- unlist(metadata$chunks)
+  alt_chunk_dim <- unlist(metadata$chunk_grid$configuration$chunk_shape)
 
-  for (j in seq_len(ncol(required_chunks))) {
-    index_in_result[[j]] <- which(chunk_idx[[j]] == required_chunks[i, j])
+  # FIXME: deal with this by rewriting the chunk grid in metadata after we supported
+  # non-regular chunk grid
+  for (j in seq_along(current_chunk_index)) {
+    index_in_result[[j]] <- which(chunk_idx[[j]] == current_chunk_index[j])
     ## are we requesting values outside the array due to overhanging chunks?
     outside_extent <- index_in_result[[j]] > metadata$shape[[j]]
     if (any(outside_extent)) {
@@ -109,14 +134,14 @@ read_zarr_array <- function(zarr_array_path, index, s3_client) {
     }
 
     index_in_chunk[[j]] <- ((index[[j]][index_in_result[[j]]] - 1) %%
-      metadata$chunks[[j]]) +
+      metadata$chunk_grid$configuration$chunk_shape[[j]]) +
       1
   }
 
   ## read this chunk
   chunk <- read_chunk(
     zarr_array_path,
-    chunk_id = required_chunks[i, ],
+    chunk_name = current_chunk_name,
     metadata = metadata,
     s3_client = s3_client,
     alt_chunk_dim = alt_chunk_dim
@@ -152,20 +177,29 @@ read_data <- function(
       (x - 1) %/% y
     },
     index,
-    metadata$chunks,
+    metadata$chunk_grid$configuration$chunk_shape,
     SIMPLIFY = FALSE
   )
 
+  chunk_names <- .create_chunk_names(
+    required_chunks,
+    metadata
+  )
+
   ## hopefully we can eventually do this in parallel
   chunk_selections <- lapply(
-    seq_len(nrow(required_chunks)),
-    FUN = .extract_elements,
-    metadata = metadata,
-    index = index,
-    required_chunks = required_chunks,
-    zarr_array_path = zarr_array_path,
-    s3_client = s3_client,
-    chunk_idx = chunk_idx
+    seq_along(chunk_names),
+    function(i) {
+      .extract_elements(
+        current_chunk_index = required_chunks[i, ],
+        current_chunk_name = chunk_names[i],
+        metadata = metadata,
+        index = index,
+        zarr_array_path = zarr_array_path,
+        s3_client = s3_client,
+        chunk_idx = chunk_idx
+      )
+    }
   )
 
   ## predefine our array to be populated from the read chunks
@@ -189,7 +223,9 @@ read_data <- function(
 find_chunks_needed <- function(metadata, index) {
   index_chunks <- list()
   for (i in seq_along(index)) {
-    index_chunks[[i]] <- unique((index[[i]] - 1) %/% metadata$chunks[[i]])
+    index_chunks[[i]] <- unique(
+      (index[[i]] - 1) %/% metadata$chunk_grid$configuration$chunk_shape[[i]]
+    )
   }
 
   required_chunks <- expand.grid(index_chunks)
@@ -215,8 +251,7 @@ get_decompressed_chunk_size <- function(datatype, dimensions) {
 #'
 #' @param zarr_array_path A character vector of length 1, giving the path to the
 #'   Zarr array
-#' @param chunk_id A numeric vector or single data.frame row with length equal
-#'   to the number of dimensions of a chunk.
+#' @param chunk_name The name of the chunk to read.
 #' @param metadata List produced by `.read_array_metadata()` holding the contents
 #'   of the `.zarray` file. If missing this function will be called
 #'   automatically, but it is probably preferable to pass the meta data rather
@@ -237,23 +272,12 @@ get_decompressed_chunk_size <- function(datatype, dimensions) {
 #' @keywords internal
 read_chunk <- function(
   zarr_array_path,
-  chunk_id,
+  chunk_name,
   metadata,
   s3_client = NULL,
   alt_chunk_dim = NULL
 ) {
-  if (missing(metadata)) {
-    metadata <- .read_array_metadata(
-      zarr_array_path,
-      ".zarray",
-      s3_client = s3_client
-    )
-  }
-
-  dim_separator <- metadata$dimension_separator %||% "."
-  chunk_id <- paste(chunk_id, collapse = dim_separator)
-
-  chunk_file <- paste0(zarr_array_path, chunk_id)
+  chunk_file <- paste0(zarr_array_path, chunk_name)
 
   if (nzchar(Sys.getenv("RARR_DEBUG"))) {
     message(chunk_file)
@@ -283,9 +307,12 @@ read_chunk <- function(
   ## or create a new chunk based on the fill value
   if (!is.null(compressed_chunk)) {
     decompressed_chunk <- .decompress_chunk(compressed_chunk, metadata)
+
+    bytes_codec_config <- metadata$codecs[["bytes"]]$configuration %||%
+      NA_character_
     decompressed_chunk <- codec_endian_decode(
       decompressed_chunk,
-      metadata$datatype$endian,
+      bytes_codec_config,
       ifelse(
         # For unicode, nbytes actually is sizeof(int) * nchar
         metadata$datatype$base_type == "unicode",
@@ -301,7 +328,10 @@ read_chunk <- function(
     # FIXME: run array -> array codecs here
   } else {
     converted_chunk <- list(
-      "chunk_data" = array(metadata$fill_value, dim = unlist(metadata$chunks)),
+      "chunk_data" = array(
+        metadata$fill_value,
+        dim = unlist(metadata$chunk_grid$configuration$chunk_shape)
+      ),
       "warning" = 0L
     )
   }
@@ -343,25 +373,26 @@ read_chunk <- function(
   ## values or not, so we try both approaches.
   actual_chunk_size <- length(decompressed_chunk) / datatype$nbytes
   if (
-    (datatype$base_type == "py_object") ||
-      (actual_chunk_size == prod(unlist(metadata$chunks)))
+    !is.null(metadata$codecs[["vlen-utf8"]]) ||
+      (actual_chunk_size ==
+        prod(unlist(metadata$chunk_grid$configuration$chunk_shape)))
   ) {
-    chunk_dim <- unlist(metadata$chunks)
-  } else if (actual_chunk_size == prod(alt_chunk_dim)) {
-    chunk_dim <- alt_chunk_dim
+    chunk_dim <- unlist(metadata$chunk_grid$configuration$chunk_shape)
   } else {
-    stop("Decompressed data doesn't match expected chunk size.")
+    chunk_dim <- alt_chunk_dim
   }
 
   if (datatype$base_type == "string") {
-    converted_chunk <- .format_string(decompressed_chunk, datatype)
-    dim(converted_chunk[[1]]) <- chunk_dim
+    if (!is.null(metadata$codecs[["vlen-utf8"]])) {
+      converted_chunk <- .format_object(decompressed_chunk, metadata, datatype)
+      dim(converted_chunk[[1]]) <- chunk_dim
+    } else {
+      converted_chunk <- .format_string(decompressed_chunk, datatype)
+      dim(converted_chunk[[1]]) <- chunk_dim
+    }
   } else if (datatype$base_type == "unicode") {
     converted_chunk <- .format_unicode(decompressed_chunk, datatype)
     dim(converted_chunk[[1]]) <- chunk_dim
-  } else if (datatype$base_type == "py_object") {
-    converted_chunk <- .format_object(decompressed_chunk, metadata, datatype)
-    dim(converted_chunk[[1]]) <- chunk_dim
   } else {
     output_type <- switch(
       datatype$base_type,
@@ -381,10 +412,12 @@ read_chunk <- function(
     )
   }
 
-  if (metadata$order == "C") {
+  if (!is.null(metadata$codecs$transpose)) {
+    transpose_config <- metadata$codecs[["transpose"]]$configuration
+    # FIXME: R is already F ordered, so we reverse the order in config
     converted_chunk[[1]] <- codec_transpose_encode(
       converted_chunk[[1]],
-      rev(seq_along(chunk_dim))
+      rev(unlist(transpose_config$order)) + 1L
     )
   }
 
@@ -411,14 +444,17 @@ read_chunk <- function(
 #' @importFrom utils tail
 #' @keywords internal
 .decompress_chunk <- function(compressed_chunk, metadata) {
-  decompressor <- metadata$compressor$id
+  decompressor <- intersect(
+    names(metadata$codecs),
+    c("blosc", "zlib", "gzip", "bz2", "lzma", "lz4", "zstd")
+  )
   datatype <- metadata$datatype
   buffer_size <- get_decompressed_chunk_size(
     datatype,
-    dimensions = metadata$chunks
+    dimensions = metadata$chunk_grid$configuration$chunk_shape
   )
 
-  if (is.null(decompressor)) {
+  if (length(decompressor) == 0) {
     decompressed_chunk <- compressed_chunk
   } else if (decompressor == "blosc") {
     decompressed_chunk <- .Call(
