@@ -144,6 +144,7 @@ zarr_overview <- function(zarr_array_path, s3_client, as_data_frame = FALSE) {
   # FIXME: once the data reading/writing code has been updated to use v3, this
   # should move to .read_array_metadata()
   if (array_metadata$zarr_format == 2) {
+    array_metadata$datatype <- .parse_datatype(array_metadata$dtype)
     array_metadata <- .convert_metadata_version(
       array_metadata,
       version_from = 2,
@@ -299,6 +300,7 @@ zarr_overview <- function(zarr_array_path, s3_client, as_data_frame = FALSE) {
     metadata$datatype <- .parse_datatype(metadata$dtype)
     metadata <- .update_fill_value(metadata, metadata$datatype)
   } else if (metadata$zarr_format == 3) {
+    metadata$datatype <- .parse_datatype_v3(metadata$data_type)
     # We shouldn't have any case where x$name is NULL since the v3 spec states
     # 'name' MUST be a plain string.
     names(metadata$codecs) <- vapply(
@@ -306,6 +308,24 @@ zarr_overview <- function(zarr_array_path, s3_client, as_data_frame = FALSE) {
       function(x) x$name,
       character(1)
     )
+    if (length(metadata$shape) == 0) {
+      # Empty tuple in shape means we are dealing with a scalar.
+      metadata$shape <- 1
+      metadata$chunk_grid <- list(
+        name = "regular",
+        configuration = list(chunk_shape = 1)
+      )
+    }
+    # This needs to happen after we address the scalar edge case
+    if (is.null(metadata$codecs[["transpose"]])) {
+      # We need to make sure this is always present because we do the
+      # reverse of what this codec is telling us (since R uses F-order).
+      # So even when using the implicit default, we need to add it here.
+      metadata$codecs[["transpose"]] <- list(
+        name = "transpose",
+        configuration = list(order = seq_along(metadata$shape) - 1)
+      )
+    }
   } else {
     stop(
       "Unsupported Zarr format version: ",
