@@ -6,9 +6,15 @@
     )
   }
 
-  dt <- .parse_datatype(metadata$dtype)
+  dt <- metadata$datatype
+
+  if (length(metadata$shape) == 0) {
+    # Empty tuple in shape means we are dealing with a scalar.
+    metadata$shape <- metadata$chunks <- 1
+  }
 
   metadata_v3 <- list(
+    zarr_format = 2L,
     datatype = dt,
     shape = metadata$shape,
     chunk_grid = list(
@@ -22,6 +28,7 @@
         separator = metadata$dimension_separator %||% "."
       )
     ),
+    # FIXME: we get invalid types from this. For example, unicode no longer exists
     data_type = paste0(dt$base_type, 8 * dt$nbytes),
     fill_value = metadata$fill_value,
     codecs = list(
@@ -34,8 +41,9 @@
         configuration = list(
           order = switch(
             metadata$order,
-            "F" = seq_along(metadata$shape) - 1, # zero indexed
-            "C" = rev(seq_along(metadata$shape)) - 1
+            # default in numpy is "C"
+            "C" = seq_along(metadata$shape) - 1, # zero indexed
+            "F" = rev(seq_along(metadata$shape)) - 1
           )
         )
       )
@@ -44,13 +52,24 @@
   )
 
   if (!is.null(metadata$compressor$id)) {
-    metadata_v3$codecs[[metadata$compressor$id]] <-
-      list(
-        name = metadata$compressor$id,
-        configuration = list(
-          metadata$compressor[names(metadata$compressor) != "id"]
-        )
+    metadata_v3$codecs[[metadata$compressor$id]] <- list(
+      name = metadata$compressor$id,
+      configuration = list(
+        metadata$compressor[names(metadata$compressor) != "id"]
       )
+    )
+  }
+
+  for (filter in metadata$filters) {
+    metadata_v3$codecs[[filter$id]] <- list(
+      name = filter$id
+    )
+  }
+
+  if (!is.null(metadata_v3$codecs[["vlen-utf8"]])) {
+    # In v3, vlen-utf8 applies to 'string' type
+    metadata_v3$data_type <- "string"
+    metadata_v3$datatype$base_type <- "string"
   }
 
   return(metadata_v3)
