@@ -55,3 +55,25 @@ test_that("i32 zarr array can be written", {
     roundtrip_i32_array
   )
 })
+
+test_that("v2 and v3 return identical results", {
+  zarr_v2 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "int32.zarr",
+    package = "Rarr"
+  )
+  zarr_v3 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "int32_v3.zarr",
+    package = "Rarr"
+  )
+
+  expect_no_condition(i32_v2 <- read_zarr_array(zarr_v2))
+  expect_no_condition(i32_v3 <- read_zarr_array(zarr_v3))
+
+  expect_identical(i32_v2, i32_v3)
+})
