@@ -41,3 +41,25 @@ test_that("uint32 zarr arrays can be read correctly", {
   expect_true(all(is.na(column_major)))
   expect_true(all(is.na(row_major)))
 })
+
+test_that("v2 and v3 return identical results", {
+  zarr_v2 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "uint32.zarr",
+    package = "Rarr"
+  )
+  zarr_v3 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "uint32_v3.zarr",
+    package = "Rarr"
+  )
+
+  u32_v2 <- read_zarr_array(zarr_v2)
+  u32_v3 <- read_zarr_array(zarr_v3)
+
+  expect_identical(u32_v2, u32_v3)
+})
