@@ -39,3 +39,25 @@ test_that("uint64 zarr arrays produce NA for out-of-range values", {
   )
   expect_true(all(is.na(column_major)))
 })
+
+test_that("v2 and v3 return identical results", {
+  zarr_v2 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "uint64.zarr",
+    package = "Rarr"
+  )
+  zarr_v3 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "uint64_v3.zarr",
+    package = "Rarr"
+  )
+
+  u64_v2 <- read_zarr_array(zarr_v2)
+  u64_v3 <- read_zarr_array(zarr_v3)
+
+  expect_identical(u64_v2, u64_v3)
+})
